@@ -1,5 +1,4 @@
 /**********************************************************************
- * $Id: liblwgeom.h.in 13391 2015-03-23 16:34:30Z strk $
  *
  * PostGIS - Spatial Types for PostgreSQL
  *
@@ -21,6 +20,14 @@
 #include <stdint.h>
 #include "proj_api.h"
 
+#if defined(PJ_VERSION) && PJ_VERSION >= 490
+/* Enable new geodesic functions */
+#define PROJ_GEODESIC 1
+#else
+/* Use the old (pre-2.2) geodesic functions */
+#define PROJ_GEODESIC 0
+#endif
+
 /**
 * @file liblwgeom.h
 *
@@ -40,10 +47,13 @@
 /**
  * liblwgeom versions
  */
-#define LIBLWGEOM_VERSION "2.1.8"
+#define LIBLWGEOM_VERSION "2.2.0"
 #define LIBLWGEOM_VERSION_MAJOR "2"
-#define LIBLWGEOM_VERSION_MINOR "1"
-#define LIBLWGEOM_GEOS_VERSION "34"
+#define LIBLWGEOM_VERSION_MINOR "2"
+#define LIBLWGEOM_GEOS_VERSION "35"
+
+/** Return lwgeom version string (not to be freed) */
+const char* lwgeom_version(void);
 
 /**
 * Return types for functions with status returns.
@@ -84,6 +94,14 @@
 #define WKBSRIDFLAG 0x20000000
 #define WKBBBOXFLAG 0x10000000
 
+/** Ordinate names */
+typedef enum LWORD_T {
+  LWORD_X = 0,
+  LWORD_Y = 1,
+  LWORD_Z = 2,
+  LWORD_M = 3
+} LWORD;
+
 /**********************************************************************
 ** Spherical radius.
 ** Moritz, H. (1980). Geodetic Reference System 1980, by resolution of 
@@ -160,6 +178,10 @@
 */
 #define SRID_DEFAULT 4326
 
+#ifndef __GNUC__
+# define __attribute__(x)
+#endif
+
 /**
 * Return a valid SRID from an arbitrary integer
 * Raises a notice if what comes out is different from
@@ -168,6 +190,8 @@
 */
 extern int clamp_srid(int srid);
 
+/* Raise an lwerror if srids do not match */
+void error_if_srid_mismatch(int srid1, int srid2);
 
 /**
  * Global functions for memory/logging handlers.
@@ -175,7 +199,10 @@ extern int clamp_srid(int srid);
 typedef void* (*lwallocator)(size_t size);
 typedef void* (*lwreallocator)(void *mem, size_t size);
 typedef void (*lwfreeor)(void* mem);
-typedef void (*lwreporter)(const char* fmt, va_list ap);
+typedef void (*lwreporter)(const char* fmt, va_list ap)
+  __attribute__ (( format(printf, 1, 0) ));
+typedef void (*lwdebuglogger)(int level, const char* fmt, va_list ap)
+  __attribute__ (( format(printf, 2,0) ));
 
 /**
 * Install custom memory management and error handling functions you want your
@@ -187,38 +214,39 @@ extern void lwgeom_set_handlers(lwallocator allocator,
         lwreallocator reallocator, lwfreeor freeor, lwreporter errorreporter,
         lwreporter noticereporter);
 
-/**
- * Write a notice out to the notice handler.
- *
- * Uses standard printf() substitutions.
- * Use for messages you always want output.
- * For debugging, use LWDEBUG() or LWDEBUGF().
- * @ingroup logging
- */
-void lwnotice(const char *fmt, ...);
+extern void lwgeom_set_debuglogger(lwdebuglogger debuglogger);
 
 /**
- * Write a notice out to the error handler.
+ * Request interruption of any running code
  *
- * Uses standard printf() substitutions.
- * Use for errors you always want output.
- * For debugging, use LWDEBUG() or LWDEBUGF().
- * @ingroup logging
+ * Safe for use from signal handlers
+ *
+ * Interrupted code will (as soon as it finds out
+ * to be interrupted) cleanup and return as soon as possible.
+ *
+ * The return value from interrupted code is undefined,
+ * it is the caller responsibility to not take it in consideration.
+ * 
  */
-void lwerror(const char *fmt, ...);
+extern void lwgeom_request_interrupt(void);
 
+/**
+ * Cancel any interruption request
+ */
+extern void lwgeom_cancel_interrupt(void);
 
-/* TODO: move these elsewhere */
-extern int lw_vasprintf (char **result, const char *format, va_list args);
-extern int lw_asprintf
-#if __STDC__
-(char **result, const char *format, ...);
-#else
-(result, va_alist);
-char **result;
-va_dcl
-#endif
-
+/**
+ * Install a callback to be called periodically during
+ * algorithm execution. Mostly only needed on WIN32 to
+ * dispatch queued signals.
+ *
+ * The callback is invoked before checking for interrupt
+ * being requested, so you can request interruption from
+ * the callback, if you want (see lwgeom_request_interrupt).
+ *
+ */
+typedef void (lwinterrupt_callback)();
+extern lwinterrupt_callback *lwgeom_register_interrupt_callback(lwinterrupt_callback *);
 
 /******************************************************************/
 
@@ -559,6 +587,7 @@ extern LWPSURFACE *lwgeom_as_lwpsurface(const LWGEOM *lwgeom);
 extern LWTRIANGLE *lwgeom_as_lwtriangle(const LWGEOM *lwgeom);
 extern LWTIN *lwgeom_as_lwtin(const LWGEOM *lwgeom);
 extern LWGEOM *lwgeom_as_multi(const LWGEOM *lwgeom);
+extern LWGEOM *lwgeom_as_curve(const LWGEOM *lwgeom);
 
 /* Casts LW*->LWGEOM (always cast) */
 extern LWGEOM *lwtin_as_lwgeom(const LWTIN *obj);
@@ -778,6 +807,14 @@ extern const POINT2D* getPoint2d_cp(const POINTARRAY *pa, int n);
 */
 extern const POINT3DZ* getPoint3dz_cp(const POINTARRAY *pa, int n);
 
+/**
+* Returns a POINT4D pointer into the POINTARRAY serialized_ptlist, 
+* suitable for reading from. This is very high performance
+* and declared const because you aren't allowed to muck with the 
+* values, only read them.
+*/
+extern const POINT4D* getPoint4d_cp(const POINTARRAY *pa, int n);
+
 /*
  * set point N to the given value
  * NOTE that the pointarray can be of any
@@ -862,14 +899,40 @@ extern int ptarray_insert_point(POINTARRAY *pa, const POINT4D *p, int where);
 */
 extern int ptarray_remove_point(POINTARRAY *pa, int where);
 
+/**
+ * @brief Add a point in a pointarray.
+ *
+ * @param pa the source POINTARRAY
+ * @param p the point to add
+ * @param pdims number of ordinates in p (2..4)
+ * @param where to insert the point. 0 prepends, pa->npoints appends
+ *
+ * @returns a newly constructed POINTARRAY using a newly allocated buffer
+ *          for the actual points, or NULL on error.
+ */
 extern POINTARRAY *ptarray_addPoint(const POINTARRAY *pa, uint8_t *p, size_t pdims, uint32_t where);
+
+/**
+ * @brief Remove a point from a pointarray.
+ * 	@param which -  is the offset (starting at 0)
+ * @return #POINTARRAY is newly allocated
+ */
 extern POINTARRAY *ptarray_removePoint(POINTARRAY *pa, uint32_t where);
+
+/**
+ * @brief Merge two given POINTARRAY and returns a pointer
+ * on the new aggregate one.
+ * Warning: this function free the two inputs POINTARRAY
+ * @return #POINTARRAY is newly allocated
+ */
 extern POINTARRAY *ptarray_merge(POINTARRAY *pa1, POINTARRAY *pa2);
+
 extern int ptarray_is_closed(const POINTARRAY *pa);
 extern int ptarray_is_closed_2d(const POINTARRAY *pa);
 extern int ptarray_is_closed_3d(const POINTARRAY *pa);
 extern int ptarray_is_closed_z(const POINTARRAY *pa);
 extern void ptarray_longitude_shift(POINTARRAY *pa);
+extern int ptarray_isccw(const POINTARRAY *pa);
 extern void ptarray_reverse(POINTARRAY *pa);
 extern POINTARRAY* ptarray_flip_coordinates(POINTARRAY *pa);
 
@@ -890,7 +953,8 @@ extern LWGEOM* lwgeom_force_3dz(const LWGEOM *geom);
 extern LWGEOM* lwgeom_force_3dm(const LWGEOM *geom);
 extern LWGEOM* lwgeom_force_4d(const LWGEOM *geom);
 
-extern LWGEOM* lwgeom_simplify(const LWGEOM *igeom, double dist);
+extern LWGEOM* lwgeom_simplify(const LWGEOM *igeom, double dist, int preserve_collapsed);
+extern LWGEOM* lwgeom_set_effective_area(const LWGEOM *igeom, int set_area, double area);
 
 /* 
  * Force to use SFS 1.1 geometry type
@@ -944,6 +1008,12 @@ extern int lwcurvepoly_add_ring(LWCURVEPOLY *poly, LWGEOM *ring);
 */
 extern int lwcompound_add_lwgeom(LWCOMPOUND *comp, LWGEOM *geom);
 
+/**
+* Construct an equivalent compound curve from a linestring. 
+* Compound curves can have linear components, so this works fine 
+*/
+extern LWCOMPOUND* lwcompound_construct_from_lwline(const LWLINE *lwpoly);
+
 /**
 * Construct an equivalent curve polygon from a polygon. Curve polygons
 * can have linear rings as their rings, so this works fine (in theory?)
@@ -1059,29 +1129,33 @@ extern float  next_float_up(double d);
 extern double next_double_down(float d);
 extern double next_double_up(float d);
 
-/* general utilities
-	2D*/
-extern double distance2d_pt_pt(const POINT2D *p1, const POINT2D *p2);
-extern double distance2d_pt_seg(const POINT2D *p, const POINT2D *A, const POINT2D *B);
+/* general utilities 2D */
+extern double  distance2d_pt_pt(const POINT2D *p1, const POINT2D *p2);
 extern double  distance2d_sqr_pt_pt(const POINT2D *p1, const POINT2D *p2);
+extern double  distance2d_pt_seg(const POINT2D *p, const POINT2D *A, const POINT2D *B);
 extern double  distance2d_sqr_pt_seg(const POINT2D *p, const POINT2D *A, const POINT2D *B);
-extern LWGEOM *lw_dist2d_distancepoint(LWGEOM *lw1, LWGEOM *lw2,int srid,int mode);
-extern LWGEOM *lw_dist2d_distanceline(LWGEOM *lw1, LWGEOM *lw2,int srid,int mode);
-extern double lwgeom_mindistance2d(LWGEOM *lw1, LWGEOM *lw2);
-extern double lwgeom_mindistance2d_tolerance(LWGEOM *lw1, LWGEOM *lw2, double tolerance);
-extern double lwgeom_maxdistance2d(LWGEOM *lw1, LWGEOM *lw2);
-extern double lwgeom_maxdistance2d_tolerance(LWGEOM *lw1, LWGEOM *lw2, double tolerance);
-/*
-	3D*/
+extern LWGEOM* lwgeom_closest_line(const LWGEOM *lw1, const LWGEOM *lw2);
+extern LWGEOM* lwgeom_furthest_line(const LWGEOM *lw1, const LWGEOM *lw2);
+extern LWGEOM* lwgeom_closest_point(const LWGEOM *lw1, const LWGEOM *lw2);
+extern LWGEOM* lwgeom_furthest_point(const LWGEOM *lw1, const LWGEOM *lw2);
+extern double  lwgeom_mindistance2d(const LWGEOM *lw1, const LWGEOM *lw2);
+extern double  lwgeom_mindistance2d_tolerance(const LWGEOM *lw1, const LWGEOM *lw2, double tolerance);
+extern double  lwgeom_maxdistance2d(const LWGEOM *lw1, const LWGEOM *lw2);
+extern double  lwgeom_maxdistance2d_tolerance(const LWGEOM *lw1, const LWGEOM *lw2, double tolerance);
+
+/* 3D */
 extern double distance3d_pt_pt(const POINT3D *p1, const POINT3D *p2);
 extern double distance3d_pt_seg(const POINT3D *p, const POINT3D *A, const POINT3D *B);
-extern LWGEOM *lw_dist3d_distancepoint(LWGEOM *lw1, LWGEOM *lw2,int srid,int mode);
-extern LWGEOM *lw_dist3d_distanceline(LWGEOM *lw1, LWGEOM *lw2,int srid,int mode);
-extern double lwgeom_mindistance3d(LWGEOM *lw1, LWGEOM *lw2);
-extern double lwgeom_mindistance3d_tolerance(LWGEOM *lw1, LWGEOM *lw2, double tolerance);
-extern double lwgeom_maxdistance3d(LWGEOM *lw1, LWGEOM *lw2);
-extern double lwgeom_maxdistance3d_tolerance(LWGEOM *lw1, LWGEOM *lw2, double tolerance);
 
+extern LWGEOM* lwgeom_furthest_line_3d(LWGEOM *lw1, LWGEOM *lw2);
+extern LWGEOM* lwgeom_closest_line_3d(const LWGEOM *lw1, const LWGEOM *lw2);
+extern LWGEOM* lwgeom_closest_point_3d(const LWGEOM *lw1, const LWGEOM *lw2);
+
+
+extern double lwgeom_mindistance3d(const LWGEOM *lw1, const LWGEOM *lw2);
+extern double lwgeom_mindistance3d_tolerance(const LWGEOM *lw1, const LWGEOM *lw2, double tolerance);
+extern double lwgeom_maxdistance3d(const LWGEOM *lw1, const LWGEOM *lw2);
+extern double lwgeom_maxdistance3d_tolerance(const LWGEOM *lw1, const LWGEOM *lw2, double tolerance);
 
 extern double lwgeom_area(const LWGEOM *geom);
 extern double lwgeom_length(const LWGEOM *geom);
@@ -1089,10 +1163,16 @@ extern double lwgeom_length_2d(const LWGEOM *geom);
 extern double lwgeom_perimeter(const LWGEOM *geom);
 extern double lwgeom_perimeter_2d(const LWGEOM *geom);
 extern void lwgeom_affine(LWGEOM *geom, const AFFINE *affine);
+extern void lwgeom_scale(LWGEOM *geom, const POINT4D *factors);
 extern int lwgeom_dimension(const LWGEOM *geom);
 
-extern LWPOINT* lwline_get_lwpoint(LWLINE *line, int where);
-extern LWPOINT* lwcircstring_get_lwpoint(LWCIRCSTRING *circ, int where);
+extern LWPOINT* lwline_get_lwpoint(const LWLINE *line, int where);
+extern LWPOINT* lwcircstring_get_lwpoint(const LWCIRCSTRING *circ, int where);
+
+extern LWPOINT* lwcompound_get_startpoint(const LWCOMPOUND *lwcmp);
+extern LWPOINT* lwcompound_get_endpoint(const LWCOMPOUND *lwcmp);
+extern LWPOINT* lwcompound_get_lwpoint(const LWCOMPOUND *lwcmp, int where);
+
 
 extern double ptarray_length_2d(const POINTARRAY *pts);
 extern double ptarray_length(const POINTARRAY *pts);
@@ -1237,7 +1317,7 @@ extern LWPOINT *lwpoint_make4d(int srid, double x, double y, double z, double m)
 extern LWPOINT *lwpoint_make(int srid, int hasz, int hasm, const POINT4D *p);
 extern LWLINE *lwline_from_lwgeom_array(int srid, uint32_t ngeoms, LWGEOM **geoms);
 extern LWLINE *lwline_from_ptarray(int srid, uint32_t npoints, LWPOINT **points); /* TODO: deprecate */
-extern LWLINE *lwline_from_lwmpoint(int srid, LWMPOINT *mpoint);
+extern LWLINE *lwline_from_lwmpoint(int srid, const LWMPOINT *mpoint);
 extern LWLINE *lwline_addpoint(LWLINE *line, LWPOINT *point, uint32_t where);
 extern LWLINE *lwline_removepoint(LWLINE *line, uint32_t which);
 extern void lwline_setPoint4d(LWLINE *line, uint32_t which, POINT4D *newpoint);
@@ -1310,6 +1390,32 @@ extern LWCOLLECTION* lwgeom_locate_between(const LWGEOM *lwin, double from, doub
 */
 extern double lwgeom_interpolate_point(const LWGEOM *lwin, const LWPOINT *lwpt);
 
+/**
+* Find the time of closest point of approach
+*
+* @param mindist if not null will be set to the minimum distance between
+*                the trajectories at the closest point of approach.
+*
+* @return the time value in which the minimum distance was reached, -1
+*         if inputs are invalid (lwerror is called in that case),
+*         -2 if the trajectories do not share any point in time.
+*/
+extern double lwgeom_tcpa(const LWGEOM *g1, const LWGEOM *g2, double *mindist);
+
+/**
+* Is the closest point of approach within a distance ?
+*
+* @return LW_TRUE or LW_FALSE
+*/
+extern int lwgeom_cpa_within(const LWGEOM *g1, const LWGEOM *g2, double maxdist);
+
+/**
+* Return LW_TRUE or LW_FALSE depending on whether or not a geometry is
+* a linestring with measure value growing from start to end vertex
+*/
+extern int lwgeom_is_trajectory(const LWGEOM *geom);
+extern int lwline_is_trajectory(const LWLINE *geom);
+
 /*
  * Ensure every segment is at most 'dist' long.
  * Returned LWGEOM might is unchanged if a POINT.
@@ -1368,6 +1474,17 @@ LWCOLLECTION* lwgeom_clip_to_ordinate_range(const LWGEOM *lwin, char ordinate, d
 #define IS_DEGREE(x) ((x) & LW_GML_IS_DEGREE)
 /** @} */
 
+/**
+ * Macros for specifying X3D options. 
+ * @{
+ */
+/** For flip X/Y coordinates to Y/X */
+#define LW_X3D_FLIP_XY     (1<<0)
+#define LW_X3D_USE_GEOCOORDS     (1<<1)
+#define X3D_USE_GEOCOORDS(x) ((x) & LW_X3D_USE_GEOCOORDS)
+
+
+
 extern char* lwgeom_to_gml2(const LWGEOM *geom, const char *srs, int precision, const char *prefix);
 extern char* lwgeom_extent_to_gml2(const LWGEOM *geom, const char *srs, int precision, const char *prefix);
 /**
@@ -1379,6 +1496,7 @@ extern char* lwgeom_to_kml2(const LWGEOM *geom, int precision, const char *prefi
 extern char* lwgeom_to_geojson(const LWGEOM *geo, char *srs, int precision, int has_bbox);
 extern char* lwgeom_to_svg(const LWGEOM *geom, int precision, int relative);
 extern char* lwgeom_to_x3d3(const LWGEOM *geom, char *srs, int precision, int opts, const char *defid);
+extern char* lwgeom_to_encoded_polyline(const LWGEOM *geom, int precision);
 
 /**
  * Create an LWGEOM object from a GeoJSON representation
@@ -1391,6 +1509,13 @@ extern char* lwgeom_to_x3d3(const LWGEOM *geom, char *srs, int precision, int op
  */
 extern LWGEOM* lwgeom_from_geojson(const char *geojson, char **srs);
 
+/**
+ * Create an LWGEOM object from an Encoded Polyline representation
+ *
+ * @param encodedpolyline the Encoded Polyline input
+ */
+extern LWGEOM* lwgeom_from_encoded_polyline(const char *encodedpolyline, int precision);
+
 /**
 * Initialize a spheroid object for use in geodetic functions.
 */
@@ -1446,13 +1571,30 @@ extern int lwgeom_covers_lwgeom_sphere(const LWGEOM *lwgeom1, const LWGEOM *lwge
 /**
 * Remove repeated points!
 */
-extern LWGEOM* lwgeom_remove_repeated_points(LWGEOM *in);
+extern LWGEOM* lwgeom_remove_repeated_points(LWGEOM *in, double tolerance);
 
 extern char lwtriangle_is_repeated_points(LWTRIANGLE *triangle);
 
+/**
+ * Swap ordinate values in every vertex of the geometry.
+ *
+ * Ordinates to swap are specified using an index with meaning:
+ * 0=x, 1=y, 2=z, 3=m
+ *
+ * Swapping an existing ordinate with an unexisting one results
+ * in undefined value being written in the existing ordinate.
+ * Caller should verify and prevent such calls.
+ *
+ * Availability: 2.2.0
+ */
+extern void lwgeom_swap_ordinates(LWGEOM *in, LWORD o1, LWORD o2);
+
 /**
 * Reverse the X and Y coordinate order. Useful for geometries in lat/lon order
 * than need to be converted to lon/lat order.
+*
+* NOTE: uses lwgeom_swap_ordinates internally,
+*       kept for backward compatibility
 */
 extern LWGEOM* lwgeom_flip_coordinates(LWGEOM *in);
 
@@ -1626,6 +1768,11 @@ extern int gbox_overlaps(const GBOX *g1, const GBOX *g2);
 */
 extern int gbox_overlaps_2d(const GBOX *g1, const GBOX *g2);
 
+/**
+* Return #LW_TRUE if the first #GBOX contains the second on the 2d plane, #LW_FALSE otherwise. 
+*/
+extern int  gbox_contains_2d(const GBOX *g1, const GBOX *g2);
+
 /**
 * Copy the values of original #GBOX into duplicate.
 */
@@ -1642,6 +1789,17 @@ extern size_t gbox_serialized_size(uint8_t flags);
 */
 extern int gbox_same(const GBOX *g1, const GBOX *g2);
 
+/**
+* Check if 2 given GBOX are the same in x and y
+*/
+extern int gbox_same_2d(const GBOX *g1, const GBOX *g2);
+
+/**
+* Check if two given GBOX are the same in x and y, or would round to the same
+* GBOX in x and if serialized in GSERIALIZED
+*/
+extern int gbox_same_2d_float(const GBOX *g1, const GBOX *g2);
+
 /**
  * Round given GBOX to float boundaries
  *
@@ -1777,6 +1935,17 @@ LWGEOM_UNPARSER_RESULT;
 #define WKT_SFSQL 0x02
 #define WKT_EXTENDED 0x04
 
+
+/*
+** Variants available for TWKB
+*/
+#define TWKB_BBOX 0x01 /* User wants bboxes */
+#define TWKB_SIZE 0x02 /* User wants sizes */
+#define TWKB_ID 0x04 /* User wants id */
+#define TWKB_NO_TYPE 0x10 /* No type because it is a sub geoemtry */
+#define TWKB_NO_ID 0x20 /* No ID because it is a subgeoemtry */
+#define TWKB_DEFAULT_PRECISION 0 /* Aim for 1m (or ft) rounding by default */
+
 /*
 ** New parsing and unparsing functions.
 */
@@ -1801,7 +1970,6 @@ extern uint8_t*  lwgeom_to_wkb(const LWGEOM *geom, uint8_t variant, size_t *size
 */
 extern char*   lwgeom_to_hexwkb(const LWGEOM *geom, uint8_t variant, size_t *size_out);
 
-
 /**
 * @param lwgeom geometry to convert to EWKT
 */
@@ -1809,10 +1977,13 @@ extern char *lwgeom_to_ewkt(const LWGEOM *lwgeom);
 
 /**
  * @param check parser check flags, see LW_PARSER_CHECK_* macros
+ * @param size length of WKB byte buffer
+ * @param wkb WKB byte buffer
  */
 extern LWGEOM* lwgeom_from_wkb(const uint8_t *wkb, const size_t wkb_size, const char check);
 
 /**
+ * @param wkt WKT string
  * @param check parser check flags, see LW_PARSER_CHECK_* macros
  */
 extern LWGEOM* lwgeom_from_wkt(const char *wkt, const char check);
@@ -1842,13 +2013,32 @@ extern void lwfree(void *mem);
 /* Utilities */
 extern char *lwmessage_truncate(char *str, int startpos, int endpos, int maxlength, int truncdirection);
 
+/*
+* TWKB functions
+*/
+
+/**
+ * @param check parser check flags, see LW_PARSER_CHECK_* macros
+ * @param size parser check flags, see LW_PARSER_CHECK_* macros
+ */
+extern LWGEOM* lwgeom_from_twkb(uint8_t *twkb, size_t twkb_size, char check);
+
+/**
+ * @param geom input geometry
+ * @param variant what variations on TWKB are requested?
+ * @param twkb_size returns the length of the output TWKB in bytes if set
+ */
+extern uint8_t* lwgeom_to_twkb(const LWGEOM *geom, uint8_t variant, int8_t precision_xy, int8_t precision_z, int8_t precision_m, size_t *twkb_size);
+
+extern uint8_t* lwgeom_to_twkb_with_idlist(const LWGEOM *geom, int64_t *idlist, uint8_t variant, int8_t precision_xy, int8_t precision_z, int8_t precision_m, size_t *twkb_size);
+
 /*******************************************************************************
  * SQLMM internal functions - TODO: Move into separate header files
  ******************************************************************************/
 
 int lwgeom_has_arc(const LWGEOM *geom);
-LWGEOM *lwgeom_segmentize(LWGEOM *geom, uint32_t perQuad);
-LWGEOM *lwgeom_desegmentize(LWGEOM *geom);
+LWGEOM *lwgeom_stroke(const LWGEOM *geom, uint32_t perQuad);
+LWGEOM *lwgeom_unstroke(const LWGEOM *geom);
 
 /*******************************************************************************
  * GEOS proxy functions on LWGEOM
@@ -1865,6 +2055,10 @@ LWGEOM *lwgeom_intersection(const LWGEOM *geom1, const LWGEOM *geom2);
 LWGEOM *lwgeom_difference(const LWGEOM *geom1, const LWGEOM *geom2);
 LWGEOM *lwgeom_symdifference(const LWGEOM* geom1, const LWGEOM* geom2);
 LWGEOM *lwgeom_union(const LWGEOM *geom1, const LWGEOM *geom2);
+LWGEOM *lwgeom_linemerge(const LWGEOM *geom1);
+LWGEOM *lwgeom_unaryunion(const LWGEOM *geom1);
+LWGEOM *lwgeom_clip_by_rect(const LWGEOM *geom1, double x0, double y0, double x1, double y1);
+LWCOLLECTION *lwgeom_subdivide(const LWGEOM *geom, int maxvertices);
 
 /**
  * Snap vertices and segments of a geometry to another using a given tolerance.
@@ -1902,6 +2096,14 @@ LWGEOM* lwgeom_sharedpaths(const LWGEOM* geom1, const LWGEOM* geom2);
  */
 LWGEOM* lwgeom_offsetcurve(const LWLINE *lwline, double size, int quadsegs, int joinStyle, double mitreLimit);
 
+/*
+ * Return true if the input geometry is "simple" as per OGC defn.
+ *
+ * @return 1 if simple, 0 if non-simple, -1 on exception (lwerror is called
+ *         in that case)
+ */
+int lwgeom_is_simple(const LWGEOM *lwgeom);
+
 
 /*******************************************************************************
  * PROJ4-dependent extra functions on LWGEOM
@@ -1948,15 +2150,11 @@ LWGEOM* lwgeom_buildarea(const LWGEOM *geom) ;
 LWGEOM* lwgeom_make_valid(LWGEOM* geom);
 
 /*
- * Split polygon by line, line by line, line by point.
+ * Split (multi)polygon by line; (multi)line by (multi)line,
+ * (multi)point or (multi)polygon boundary.
  *
- * Returns all components as a collection.
- * First element of the collection is always the part which
- * remains after the cut, while the second element is the
- * part which has been cut out. We arbitrarely take the part
- * on the *right* of cut lines as the part which has been cut out.
- * For a line cut by a point the part which remains is the one
- * from start of the line to the cut point.
+ * Collections are accepted as first argument.
+ * Returns all obtained pieces as a collection.
  */
 LWGEOM* lwgeom_split(const LWGEOM* lwgeom_in, const LWGEOM* blade_in);
 
